/*
 * DoubleNumberField.java
 *
 * Created on April 11, 2002, 12:56 PM
 */

package BioFit;

/**
 *
 * @author  Jonathan Jackson
 * @version
 */
import javax.swing.*;
import javax.swing.text.*;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;

public class DoubleNumberField extends JTextField {
    private NumberFormat integerFormatter;
    
    public DoubleNumberField(double value, int columns) {
        super(columns);
        integerFormatter = NumberFormat.getNumberInstance(Locale.US);
        integerFormatter.setParseIntegerOnly(false);
        setValue(value);
    }
    
    public double getValue() {
        double retVal = 0;
        try {
            retVal = integerFormatter.parse(getText()).doubleValue();
        } catch (ParseException e) {
            // This should never happen because insertString allows
            // only properly formatted data to get in the field.
            System.out.println("DoubleNumberField had invalid Field.");
        }
        return retVal;
    }
    
    public void setValue(double value) {
        //setText(integerFormatter.format(value));
        setText("" + value);
    }
    
    protected Document createDefaultModel() {
        return new WholeNumberDocument();
    }
    
    protected class WholeNumberDocument extends PlainDocument {
        public void insertString(int offs,
        String str,
        AttributeSet a)
        throws BadLocationException {
            char[] source = str.toCharArray();
            char[] result = new char[source.length];
            int j = 0;
            
            String str2 = DoubleNumberField.this.getText();
            int periods = str2.indexOf(".");
            int Es = str2.indexOf("E");
            int negatives = str2.indexOf("-");
            
            for (int i = 0; i < result.length; i++) {
                if (Character.isDigit(source[i])) {
                    result[j++] = source[i];
                } else if(("" + source[i]).compareTo(".") == 0 &&
                periods == -1) {
                    result[j++] = source[i];
                    periods++;
                } else if(("" + source[i]).compareTo("E") == 0 &&
                Es == -1) {
                    result[j++] = source[i];
                    Es++;
                } else if(("" + source[i]).compareTo("-") == 0 &&
                negatives == -1) {
                    result[j++] = source[i];
                    negatives++;
                } else {   //Don't copy it.
                }
            }
            super.insertString(offs, new String(result, 0, j), a);
        }
    }
}
