/*
 * GraphWindow.java
 *
 * Created on April 10, 2002, 7:58 PM
 */

package BioFit;

/**
 *
 * @author  Jonathan Jackson
 * @version
 */

import javax.swing.*;
import java.awt.image.BufferedImage;
import java.util.Vector;
import java.awt.*;
import java.awt.geom.AffineTransform;


class GraphWindow extends JComponent {
    BufferedImage graphImage;   //Image of the graph
    int x, y;         //Size of the grid in pixels
    double xInterval;    //Spacing of each marking
    double yInterval;
    double xNumPixels;
    double yNumPixels;
    double xLength, yLength;
    double xStart, yStart, xEnd, yEnd;  //Start and end of each axis
    Vector pointImages = new Vector();  //image of a point for each line
    int xOffset, yOffset;
    String xLabel = "", yLabel = "";
    Vector points = new Vector();       //to keep track of the points
    int numLines = 0;
    
    GraphWindow(int xsize, int ysize) {
        x = xsize;
        y = ysize;
        this.setSize(x,y);
        this.setPreferredSize(new Dimension(x,y));
        this.setMaximumSize(new Dimension(x,y));
        graphImage = new BufferedImage(x, y, BufferedImage.TYPE_INT_RGB);
        Graphics2D g2 = graphImage.createGraphics();
        g2.setPaint(Color.black);
        g2.fillRect(0,0,x,y);
        repaint();
    }
    public void setXLabel(String str){
        xLabel = str;
    }
    
    public void setYLabel(String str){
        yLabel = str;
    }
    
    public void setInterval(double x, double y){
        xInterval = x;
        yInterval = y;
    }
    
    public void setStart(double x, double y) {
        xStart = x;
        yStart = y;
    }
    
    public void setEnd(double x, double y) {
        xEnd = x;
        yEnd = y;
    }
    public void setOffset(int x, int y) {
        xOffset = x;
        yOffset = y;
    }
    
    public void calibrate() {
        //Sets the pixels values.
        xNumPixels = x-xOffset;
        yNumPixels = y-yOffset;
        yLength = yEnd - yStart;
        xLength = xEnd - xStart;
    }
    
    public void clearData(){
        drawAxis();
    }
    
    
    //ClearAll removes all the points.  Ther calibration, offsets,
    //and other numbers are left intact and must by changed on their own as
    //is the line count and the pointImages.  this could be a mem like if you
    //are constantly added in new images.
    public void clearAll(){
        points.clear();
        graphImage = new BufferedImage(x, y, BufferedImage.TYPE_INT_RGB);
        Graphics2D g2 = graphImage.createGraphics();
        g2.setPaint(Color.black);
        g2.fillRect(0,0,x,y);
        repaint();
    }
    
    
    //Adds a line to the graph, retruns in the index of the line.
    public int addLine(BufferedImage image){
        pointImages.add(image);
        numLines++;
        return pointImages.size()-1;
        
    }
    //This removes all the data points that aren't on line's line.
    public void removeExcessLines(int line) {
        clearData();
        for(int i = 0; i < points.size(); i++) {
            LinePoint pnt = (LinePoint)points.get(i);
            if(pnt.line != line){
                //remove this point and set the cntr back one so we don't
                //skip the next element in the vector
                points.remove(i);
                i--;
            } else {
                //plot it and move on
                drawPoint(pnt.line, pnt.x, pnt.y);
            }
        }
    }
        
    //Removes all the points of a line from the graph.  The BufferedImage is 
    //still stored correctly in the pointImages, so you can still add points
    //to this line.
    
    public void removeLine(int lineNumber) {
        if(lineNumber < 0 || lineNumber > numLines - 1) {
            //lines not here
            return;
        }
        clearData();
        for(int i = 0; i < points.size(); i++) {
            LinePoint pnt = (LinePoint)points.get(i);
            if(pnt.line == lineNumber){
                //remove this point and set the cntr back one so we don't
                //skip the next element in the vector
                points.remove(i);
                i--;
            } else {
                //plot it and move on
                drawPoint(pnt.line, pnt.x, pnt.y);
            }
        }
    }
    
    //call removeLine on the last line added to the graph.  Returns the number
    //of that line.
    public int removeLastLine() {
        removeLine(numLines-1);
        return numLines-1;
    }
    
    public void drawPoint(int lineNumber, double xval, double yval) {
        //translate x and y to pixels.
        int xReal = (int)((xval+xStart)/xLength * xNumPixels);
        int yReal = (int)((yval+yStart)/yLength * yNumPixels);
        
        //Now we need to put the center of the image there.  Keep in mind,
        //as y values go up in pixels, it is lower on the graph, so we
        //need to invert those.
        
        BufferedImage img = (BufferedImage)pointImages.get(lineNumber);
        int height = img.getHeight();
        int width = img.getWidth();
        xReal = xReal - width/2 + xOffset;
        yReal = y - (yReal + height/2) -yOffset;
        Graphics2D g2 = graphImage.createGraphics();
        g2.drawImage(img, xReal, yReal, this);
        repaint();
    }
    
    public void addPoint(int lineNumber, double xval, double yval){
        //keeps track of every point added to
        points.add(new LinePoint(lineNumber, xval, yval));
        drawPoint(lineNumber, xval, yval);
        
    }
    
    //This clears everything on the graph and draw axis
    public void drawAxis() {
        Graphics2D g2 = graphImage.createGraphics();
        g2.setPaint(Color.black);
        g2.fillRect(0,0,x,y);
        g2.setPaint(Color.yellow);
        
        //set the Font
        g2.setFont(new Font("Times New Roman", Font.PLAIN, 10));
        
        //Draw the Labels
        
        g2.drawString(xLabel, x/2 - g2.getFontMetrics().charWidth('A') * xLabel.length()/2, y- g2.getFontMetrics().getHeight() + 3);
        
        //We have the rotate the y label 90 degrees, then draw it.
        AffineTransform saveXform = g2.getTransform();
        g2.rotate(Math.toRadians(-90), x/2, y/2);
        g2.drawString(yLabel, x/2 - g2.getFontMetrics().charWidth('A') * yLabel.length()/2, g2.getFontMetrics().getHeight() + 1);
        g2.setTransform(saveXform);
        
        //Draw the Axis
        g2.drawLine(xOffset, y - yOffset, xOffset, 0);
        g2.drawLine(xOffset, y - yOffset, x, y - yOffset);
        
        //Draw the Intervals on the x Axis
        double yval = y- (yOffset - g2.getFontMetrics().getHeight() - 4);
        
        int xPixelVal;
        int cntr = 0;
        
        while(true) {
            double xval = xStart + (double)cntr * xInterval;  //Marking value
            String xMarking = Fit.truncString(String.valueOf(xval), 3);
	    //System.out.println("xval:" + xval + "xStart:" + xStart + "xLength:" + xLength + "xNumPixels:" + xNumPixels + "xOffset:" + xOffset);
            xPixelVal = (int)((xval+xStart)/(xLength) * xNumPixels) + xOffset;  //Pixel value
	    //System.out.println("xPixelVal:" + xPixelVal);
            if(xPixelVal > x-10){  //Make sure the marking doesn't go off the screen
                break;
            } else {
                //Draw a tick mark and center the text
                g2.drawString(xMarking, xPixelVal - xMarking.length()/2 * g2.getFontMetrics().charWidth('0'), (int)yval);
                if(cntr == 0){
                    cntr++;
                    continue; //Don't draw the first tick mark
                } else{
                    g2.drawLine(xPixelVal, y-(yOffset - 2), xPixelVal, y-(yOffset +2));
                }
                cntr++;
            }
        }
        
        //Draw the Intervals on the y Axis
        int xval = xOffset - 5 * g2.getFontMetrics().charWidth('0');
        int yPixelVal;
        cntr = 0;
        
        while(true) {
            yval = yStart + (double)cntr * yInterval;  //Marking val
            String yMarking = Fit.truncString(String.valueOf(yval), 3);
            yPixelVal = (int)(y-((yval+yStart)/(yLength) * yNumPixels) - yOffset);  //Pixel value
            if(yPixelVal < 10){  //Make sure the marking doesn't go off the screen
                break;
            } else {
                //Draw a tick mark and center the text
                g2.drawString(yMarking, xval, yPixelVal + g2.getFontMetrics().getHeight()/2);
                if(cntr == 0) {
                    cntr++;
                    continue;
                } else{
                    g2.drawLine(xOffset - 2, yPixelVal, xOffset +2, yPixelVal);
                }
                cntr++;
            }
        }
        
        
        repaint();
    }
    
    public void paintComponent(Graphics g) {
        Graphics2D g2 = (Graphics2D)g;
        g2.drawImage(graphImage, 0, 0, this);
    }
    
    // simple class used to keep track of every point and its line that is added to the graph.
    public class LinePoint {
        int line;
        double x;
        double y;
        
        LinePoint(int line, double x, double y){
            this.line = line;
            this.x = x;
            this.y = y;
        }
    }
}
